%% Code for Day 6

% Add path to EEGLab and launch EEGLab before we begin (or use your
% shortcut on the shortcuts bar)

% Make sure you can access your sample data (sampleEEGdata.mat)


%% -------------------------------------------
% REVIEW Filter/Hilbert
% --------------------------------------------

    %% ASSIGNMENT: Averaging and Plotting Hilbert and comparison to Time Domain Signal--------------

    % load your sample data
    load('..\sampleEEGdata.mat'); % change directory as needed

    % Review -- mpute the ERP at each electrode. 
    ERP_data = mean(EEG.data,3); % You will use this below

    % Choose a single channel to work with
    channel2use = 'Pz';
    channel2use_idx = strcmpi(channel2use,{EEG.chanlocs.labels});

    % Reshape the single channel of data to a vector
    % 
    datavector =EEG.data(channel2use_idx,:,:);
    datavector = reshape(datavector,1,EEG.pnts*EEG.trials);

    % Single trial only 
    % datavector =squeeze(EEG.data(channel2use_idx,:,99));
    
    
    % Filter the data with a filter of your choosing
    filt_bounds = [8 13];
    [filtered_data, f_weights] = optFIR(datavector,EEG.srate, filt_bounds(1), filt_bounds(2));

    filtered_data = reshape(filtered_data,1,EEG.pnts*EEG.trials);

    % Obtain the analytic signal using the hilbert function
    as = hilbert(filtered_data);

    % Reshape the analytic signal back to times by trials
    as = reshape(as,1,EEG.pnts,EEG.trials);

    % Take the mean power across epochs to have a vector of mean power by time
    hilbert_mean = mean(abs(as).^2,3);
    
    % Extract the channel of interest from ERP
    ERP_data = ERP_data(channel2use_idx,:);
    
    % In two subplots, plot the ERP for the chosen channel and the bandpass-filtered hilbert power signal
    % Show the times
    
        times2plot = [-500 1000] ;
    times2plot_idx = dsearchn(EEG.times',times2plot');
    
    figure;
    subplot(2,1,1);
    plot(EEG.times(times2plot_idx(1):times2plot_idx(2)),ERP_data(times2plot_idx(1):times2plot_idx(2)),'-r');
    title(['TIme-domain ERP Channel ' channel2use]);
    subplot(2,1,2);
    plot(EEG.times(times2plot_idx(1):times2plot_idx(2)),hilbert_mean(times2plot_idx(1):times2plot_idx(2)),'-k');
    title(['Filter-Hilbert Power Channel ' channel2use ' for freqs: ' num2str(filt_bounds(1)) ' to ' num2str(filt_bounds(2)) ' hz']);

    % Change the frequencies above and repeat

    % Does the average filter-hilbert power represent only the phase locked
    % activity like the ERP does?
    % 
    % Instead of plotting the mean, just plot a single trial of raw data and
    % hilbert power for the same (redefine the datavector above)
    

%% -------------------------------------------
% Short-time FFT
% --------------------------------------------

    %% Data windowing
    % create hamming, hanning, and gaussian windows

    % Mike's code ------------------
    time_Centerwin = median(EEG.times); % Center of the window in ms
    % convert ms to idx
    timewinidx = round(time_Centerwin/(1000/EEG.srate));
    hamming_win = .54 - .46*cos(2*pi*(0:timewinidx-1)/(timewinidx-1));
    hann_win    = .5*(1-cos(2*pi*(0:timewinidx-1)/(timewinidx-1)));
    gaus_win = exp(-.5*(2.5*(-timewinidx/2:timewinidx/2-1)/(timewinidx/2)).^2);
    % Mike's code ------------------

    % Plot Mike's verions
    figure;
    subplot(2,1,1)
    plot(hann_win)
    hold on
    plot(hamming_win,'r')
    plot(gaus_win,'k')
    legend({'Hann';'Hamming';'Gaussian'})

    % Matlab version
    hamming_win = hamming(EEG.pnts);
    hann_win = hann(EEG.pnts);
    gaus_win = gauss(EEG.pnts,1.7); % second parameter is std Deviation Units

    % plot together
    subplot(2,1,2)
    plot(hann_win)
    hold on
    plot(hamming_win,'r')
    plot(gaus_win,'k')
    legend({'Hann';'Hamming';'Gaussian'})

    % Plot data unwindowed and windowed
    trial_of_data = EEG.data(1,:,1); % first channel, first epoch
    figure
    plot(EEG.times,trial_of_data,'color',[.8 .8 .8]);
    hold on
    plot(EEG.times,trial_of_data.*hann_win')
    plot(EEG.times,trial_of_data.*hamming_win','r')
    plot(EEG.times,trial_of_data.*gaus_win,'k')
    legend({'un-windowed'; 'Hann';'Hamming';'Gaussian'})


    %% STFFT implementation
    edit stfft.m
    
    % ASSIGNMENT: Play with window size (timewin) to see result on the plot

%% -------------------------------------------
% Multi-taper
% --------------------------------------------
    edit multitaper.m
    
    % Question: When is the use of MultiTaper most advantageous?
    
    % Note that Toolbox FieldTrip has well-developed Multi-taper options

%% -------------------------------------------
% Integration and comparison ASSIGNMENT
% --------------------------------------------
    % ASSIGNMENT: Compare STFFT, Filter-Hilbert, Wavelet, and MultiTaper --------------
    % Select one electrode, and one frequency 
    % Compute power over time at that electrode and that frequency using:
    %  Complex wavelet convolution
    %  Filter-Hilbert
    %  Short-time FFT
    %  Multi-taper
    % Create a subplot with 5 rows and plot the ERP (average of raw signal) in the first subplot
    %   and then the power time series results of these four time-frequency decomposition 
    %   in the subsequent four subplots -- and put a title on each!
    % How visually similar are the results from these methods?
    % Repeat for a very different frequency (far away from the one you chose)

    %% Use these again and again
        channel2use = 'Pz';
        channel2use_idx = strcmpi(channel2use,{EEG.chanlocs.labels});
        freq2use = 17.2;
        time_limits = [-300 800]; % we have long epochs so we'll get rid of edge effects

    %% Wavelet approach (with some help....)
        % create a Morlet wavelet
        srate = EEG.srate; % in hz
        time  = -2:1/srate:2; % best practice is to have time=0 at the center of the wavelet
        frex  = freq2use; % frequency of wavelet, in Hz
        % create complex sine wave
        sine_wave = exp( 1i*2*pi*frex.*time );
        % create Gaussian window
        s = 7 / (2*pi*frex); % this is the standard deviation of the gaussian FIXED (can explore changing this
        % Note you could use this code to "look up a different # cycles
        gaus_win  = exp( (-time.^2) ./ (2*s^2) );
        % now create Morlet wavelet
        cmw  = sine_wave .* gaus_win;
        % convolve this complex wavelet with the data
        % Get one channel
        % extract this bit of data for convenience 
        data = reshape(EEG.data(channel2use_idx,:,:),1,EEG.pnts*EEG.trials); % long vector now
        % do the convolution using Mike's code that I made into a function and
        % reshape back to trials
        as = cmw_conv(data,cmw);
        as = reshape(as,1,EEG.pnts,EEG.trials);
        % now take mean across trials of power of as 
        wavelet_power = mean(abs(as).^2,3);
        % and trim to times we wish to use without edge effects (time_limits)
        wavelet_times_idx = dsearchn(EEG.times',time_limits');
        wavelet_times = EEG.times(wavelet_times_idx(1):wavelet_times_idx(2));
        wavelet_power = wavelet_power(1,wavelet_times_idx(1):wavelet_times_idx(2));

    %% Filter-Hilbert -- can you use what you created above in the first Assignment?
        % Reshape the single channel of data to a vector
        % 
        datavector =EEG.data(channel2use_idx,:,:);
        datavector = reshape(datavector,1,EEG.pnts*EEG.trials);

        % Filter the data with a filter of your choosing
        filt_bounds = [freq2use-2 freq2use+2];
        [filtered_data, f_weights] = optFIR(datavector,EEG.srate, filt_bounds(1), filt_bounds(2),0,1);

        filtered_data = reshape(filtered_data,1,EEG.pnts*EEG.trials);

        % Obtain the analytic signal using the hilbert function
        as = hilbert(filtered_data);

        % Reshape the analytic signal back to times by trials
        as = reshape(as,1,EEG.pnts,EEG.trials);

        % Take the mean power across epochs to have a vector of mean power by time
        hilbert_mean = mean(abs(as).^2,3);

       % and trim to times we wish to use without edge effects (time_limits)
        hilbert_times_idx = dsearchn(EEG.times',time_limits');
        hilbert_times = EEG.times(hilbert_times_idx(1):hilbert_times_idx(2));
        hilbert_power = hilbert_mean(1,hilbert_times_idx(1):hilbert_times_idx(2));


    %%  Short-time FFT -- can you see what stfft can do for you?
        frex = logspace(log10(4),log10(EEG.srate/5),25); % 4 to 1/5 sample rate
        times2save = time_limits(1):1000*1/EEG.srate:time_limits(2); % no downsampling if use the sample interval 1000*1/EEG.srate
        basetime   = [-300 -100];
        timewin    = 350; % in ms

        % convert time points to indices
        times2saveidx = dsearchn(EEG.times',times2save'); 
        % convert time window to points
        timewinpnts   = round(timewin/(1000/EEG.srate));

        % find baselinetimepoints
        baseidx = dsearchn(times2save',basetime');

        % define frequencies for FFT
        hz = linspace(0,EEG.srate/2,timewinpnts/2+1);

        % hanning window for tapering
        hannwin = .5 - .5*cos(2*pi.*linspace(0,1,timewinpnts))';

        % find logical channel index
        chanidx = strcmpi(chan2plot,{EEG.chanlocs.labels});

        % initialize output matrix
        shortFFT_tf = zeros(length(frex),length(times2save));


        % loop through time bins
        for ti=1:length(times2saveidx)

            % window and taper data, and get power spectrum
            data = bsxfun(@times, squeeze(EEG.data(chanidx,times2saveidx(ti)-floor(timewinpnts/2)+1:times2saveidx(ti)+ceil(timewinpnts/2),:)), hannwin);
            % uncomment the next line to use non-tapered data
            %data = squeeze(EEG.data(chanidx,times2saveidx(ti)-floor(timewinpnts/2)+1:times2saveidx(ti)+ceil(timewinpnts/2),:));

            y    = fft(data,timewinpnts)/(timewinpnts);
            pow  = mean(abs(y).^2,2);

            % finally, get power from closest frequency
            closestfreq = dsearchn(hz',frex');
            shortFFT_tf(:,ti) = pow(closestfreq);    
        end % end time loop
        % db-correct
        db_shortFFT_tf = 10*log10( bsxfun(@rdivide,shortFFT_tf,mean(shortFFT_tf(:,baseidx(1):baseidx(2)),2)) );

        % Find the frequency closest to our frequency of interest
        % and trim to times we wish to use without edge effects (time_limits)
        db_shortFFT_tf_times = times2save;
        db_shortFFT_tf_power = db_shortFFT_tf(freq2use_idx,:);
       
        
    %%  Multi-taper

    %% Plot it all!  Be sure to have times on X axis; 
    %  Bonus: link x axes across subplots

%% -------------------------------------------
% Power law
% --------------------------------------------

    edit powerLawTF.m

%% -------------------------------------------
% Normalization
% --------------------------------------------

    %% dB normalization (from Lecturelets code)

    activity = 1:.01:20; % activity
    baseline = 10; % baseline

    db = 10*log10(activity./ baseline );

    figure(1), clf
    plot(activity,activity,'linew',3)
    hold on
    plot(activity, db,'r','linew',3)
    legend({'"activity"','dB'})

    %% percent change
    pc = 100*( activity-baseline)./ baseline;

    figure(1), clf
    plot(activity,activity,'linew',3)
    hold on
    [hAx,hLine1,hLine2] = plotyy(activity, db,activity, pc);
    hLine1.Color = [1 0 1];
    hLine2.Color = [0 0 0];
    legend({'"activity"','dB', '%'})

%% -------------------------------------------
% Choosing a Baseline for Normalization
% --------------------------------------------
    edit whichbaseline.m

    % ASSIGNMENT: explore time windows for baseline; choose wisely and foolishly
    % Comment: Condition specific vs Condition-average




%% -------------------------------------------
% Temporal Downsampling following tf analysis
% --------------------------------------------
    edit postanalysis_ds.m
    % TF retains temporal resolution, but decreases temporal precision
